   
#seantis-overview-calendar {
    page-break-before: always;
}

div.fc {
    page-break-before: always;
}

#inline-page {
    background-color: none !important;
    border: none !important;
}

div.splitscreen {
    page-break-after: always;
    float: none;
}

.inline-reservations {
    border: none !important;
}

.reservation {
    border: none !important;
}

#content .reservation-list {
    border: none;
}

.template-reservations h2 {
    display: none;
}

.template-reservations .inline-reservations {
    padding: 0 !important;
}

.reservation-title {
    border: 1px solid black !important;
    border-width: 0 0 1px 0 !important;
    background-color: white !important;
    margin-top: 0 !important;
}

.reservation-title a {
    display: none !important;
}

#content .reservation-title h3 {
    color: black !important;
    font-weight: bold !important;
}

.reservation-timespans {
    width: 100% !important;
}

.reservation-timespans * {
    font-size: 12px !important;
}

#seantis-overview-calendar td {
    width: 80px !important;
    border: none !important;
}

#seantis-overview-calendar > * {
    -webkit-print-color-adjust: exact; 
}

div.fc > * {
    -webkit-print-color-adjust: exact;
}

#seantis-overview-calendar .event-unavailable .fc-day-content div:after {
    content: "0%" !important;
    font-size: 10px;
}

#seantis-overview-calendar .event-partly-available .fc-day-content div:after {
    content: "<100%" !important;
    font-size: 10px;
}

#seantis-overview-calendar .event-available .fc-day-content div:after {
    content: "100%" !important;
    font-size: 10px;
}

.overview-utils {
    display: none;
}

.timeframes-control, .timeframes-global-control {
    display: none;
}

#email_templates {
    display: none;
}

.monthly-report {
    font-family: Verdana, Helvetica, sans-serif;
}

.day {
    page-break-after: always;
    border: none !important;
    border-width: 0 !important;
}

.day:last-child {
    page-break-after: avoid;
}

.leftcolumn h4 {
    margin-top: 0;   
}

.monthly-report * {
  -webkit-print-color-adjust: exact;   
}

.reservation {
    font-size: 12px !important;
    border: 1px solid black !important;   
    margin-bottom: 10px !important;
}

.reservation-urls {
    display: none;   
}

.reservation-summary {
    font-size: 12px;   
}

.controlbox {
    display: none;   
}

.day h2 {
    border-width: 0 0 1px 0 !important;
}

.single-reservation-view .reservation-info {
    border: 1px solid #aaa;
    padding: 1em;
}

.single-reservation-view .reservation {
    border: none !important;
}

.single-reservation-view .splitscreen {
    width: auto;
}

.single-reservation-view .reservation-info {
    margin: 1em;
}

.single-reservation-view .reservation-title {
    margin-bottom: 1em !important;
}

.single-reservation-view .splitscreen {
    page-break-after: avoid;
}
