/* overview styles for izug basetheme */
.resource-listing {
    float: left;
    margin-top: 5px;
}

.resource-listing-entry.groupSelection,
.resource-listing-entry:hover {
    -webkit-box-shadow: none !important;
       -moz-box-shadow: none !important;
            box-shadow: none !important;
            
    background-color: #ffeeb3 !important;
}

.resource-listing-overview {
    float: right;
    max-width: 400px;
    margin-top: 5px;
}

.resource-listing-overview .fc-day.event-available,
.resource-listing-overview .fc-day.event-partly-available,
.resource-listing-overview .fc-day.event-unavailable {
    opacity: 0.8;   
}

.resource-listing-overview .fc-day.groupSelection,
.resource-listing-overview .fc-day.event-available:hover,
.resource-listing-overview .fc-day.event-partly-available:hover,
.resource-listing-overview .fc-day.event-unavailable:hover {
   color: white;
   opacity: 1.0;
}

.resource-listing-entry {
    padding: 10px;
    width: 525px;
}

.resource-listing-entry * {
    margin: 0;   
}

.resource-listing-entry p {
   color: #333;
   max-width: 500px;
}

.template-resource_listing .documentDescription {
   margin-top: 18px;
   font-weight: normal   
}

#directorySearch {
    border: none;
    margin: 0;   
}

.overviewlet ul {
    font-size: 1em;
    margin-top: 10px;   
}

.overviewlet ul li {
    background: none;
    list-style-type: none;
    background-repeat: no-repeat;
    padding-left: 0;
    margin: 0;
    font-size: 12px;
}

.overviewlet div.resourceTitle,
.overviewlet a {
    font-size: 1em;   
}

.overviewlet .resourceTitle {
    margin: 0 10px 0 0;
    font-size: 12px !important;
}


.templates-control,
.templates-global-control,
.timeframes-control,
.timeframes-global-control {
    margin: 0;
    padding: 0;
}

.templates-global-control:before,
.timeframes-global-control:before {
    margin-left: 0;   
}

#timeframes .visibleTimeframe {
    color: black !important;
}

#timeframes .hiddenTimeframe {
    color: #76797C !important;
}

#contacts .resourceItem,
#email_templates .resourceItem, 
#timeframes .resourceItem {
   margin: 0 0 10px 0;
   padding: 0;   
   border: none;
}

#seantis-overview-calendar .event-unavailable, 
#seantis-overview-calendar .event-unavailable .fc-event-skin, 
#seantis-overview-calendar .event-unavailable .fc-event-head {
    background-color: #ae1e00 !important;
}

#seantis-overview-calendar .event-available, 
#seantis-overview-calendar .event-available .fc-event-skin, 
#seantis-overview-calendar .event-available .fc-event-head {
    background-color: #0e6700 !important;
}

#seantis-overview-calendar .event-partly-available, 
#seantis-overview-calendar .event-partly-available .fc-event-skin, 
#seantis-overview-calendar .event-partly-available .fc-event-head {
    background-color: #dd7c07 !important;
}

/* fix izug basetheme's display of icons in the contentmenu */
.actionMenuContent img {
    margin-left: -22px;
    display: inline-block;
    background-color: #dedede;
}

.actionMenuContent li:hover img {
    background-color: #99bad1; 
}

#cut, #copy, #delete, #rename {
    background: none;
    margin-left: -2px;   
}

.actionMenuContent .contenttype-collection {
    background: none;
}